import { ChartMeasureRoleType, getChartCategory, getChartCategory2, getChartSeries, getEntityHierarchy, getEntityProperty, getPropertyHierarchy, getPropertyMeasure, mergeOptions, } from '@metad/ocap-core';
import { LinearGradient } from 'echarts/lib/util/graphic';
import includes from 'lodash/includes';
import isNil from 'lodash/isNil';
import isEmpty from 'lodash/isEmpty';
import assign from 'lodash/assign';
import indexOf from 'lodash/indexOf';
import isArray from 'lodash/isArray';
import maxBy from 'lodash/maxBy';
import minBy from 'lodash/minBy';
import range from 'lodash/range';
import { axisOrient, getCategoryAxis, getValueAxis } from './axis';
import { getChromaticScale } from './chromatics';
import { coordinates, gatherCoordinates } from './coordinates';
import { DecalPatterns } from './decal';
import { referenceLines } from './series';
import { stackedForMeasure } from './stacked';
import { getEChartsTooltip } from './tooltip';
import { AxisEnum } from './types';
export function cartesian(data, chartAnnotation, entityType, settings, options, type) {
    const pieCoordinates = coordinates(data, chartAnnotation, entityType, settings, options, type === 'pie' ? 'pie' : 'grid', cartesianCoordinate);
    return gatherCoordinates(pieCoordinates, type, options);
}
/**
 * 单个笛卡尔坐标系, 对应一个 grid 内的组件
 *
 * @param data 数据数组
 * @param chartAnnotation 图形注解
 * @param entityType 实体类型
 * @param settings 图形设置
 * @param options 图形属性
 * @returns
 */
export function cartesianCoordinate(data, chartAnnotation, entityType, settings, options) {
    const category = getChartCategory(chartAnnotation);
    const categoryProperty = getEntityProperty(entityType, category);
    const category2 = getChartCategory2(chartAnnotation);
    const chartSeries = getChartSeries(chartAnnotation);
    let datasets = [];
    if (chartSeries) {
        datasets = chartAnnotation.measures
            .filter(({ role }) => !includes([ChartMeasureRoleType.Tooltip, ChartMeasureRoleType.Size, ChartMeasureRoleType.Lightness], role))
            .map((measure) => {
            return stackedForMeasure(data, measure, chartAnnotation, entityType, settings, options);
        });
    }
    else {
        const seriesComponents = measuresToSeriesComponents(chartAnnotation.measures, data, entityType, settings);
        data = [...data];
        // Fill up measure null value
        if (data[0]) {
            data[0] = { ...data[0] };
            chartAnnotation.measures.forEach((measure) => {
                if (isNil(data[0][measure.measure])) {
                    data[0][measure.measure] = '-';
                }
            });
        }
        datasets = [
            {
                dataset: {
                    source: data,
                    // dimensions: [
                    //   ...chartAnnotation.dimensions.map(getPropertyName),
                    //   ...chartAnnotation.measures.map(getPropertyMeasure)
                    // ]
                },
                seriesComponents,
                tooltip: getEChartsTooltip(options?.tooltip, categoryProperty, chartAnnotation.measures.map((measure) => ({
                    measure,
                    property: getEntityProperty(entityType, measure)
                })), seriesComponents, settings.locale)
            }
        ];
    }
    const { categoryAxis, valueAxis } = getCoordinateSystem(chartAnnotation, entityType, data, options, settings.locale);
    const gridOptions = {
        name: '',
        grid: mergeOptions({
            top: 0,
            right: 0,
            bottom: 0,
            left: 0,
            containLabel: true
        }, options?.grid),
        [categoryAxis.orient]: categoryAxis.axis,
        [valueAxis.orient]: valueAxis.axis,
        visualMap: [],
        datasets: [],
        tooltip: [],
        legend: [],
        series: []
    };
    datasets.forEach(({ dataset, seriesComponents, tooltip }) => {
        gridOptions.datasets.push({
            dataset,
            series: seriesComponents.map((seriesComponent) => {
                const { series, visualMaps } = serializeSeriesComponent(dataset, seriesComponent, entityType, category, valueAxis, settings, options);
                gridOptions.visualMap.push(...visualMaps);
                if (!isNil(seriesComponent.valueAxisIndex)) {
                    series[valueAxis.orient + 'Index'] = seriesComponent.valueAxisIndex;
                }
                /**
                 * 堆积系列情况下不适用 encode, 使用的应该是 seriesLayoutBy: "row" name: "[Canada]" 与 dataset 中的行数据进行的对应
                 */
                if (isNil(series.seriesLayoutBy)) {
                    series.encode = {
                        [AxisEnum[categoryAxis.orient]]: getPropertyHierarchy(category),
                        [AxisEnum[valueAxis.orient]]: category2 ? getPropertyHierarchy(category2) : seriesComponent.measure,
                        tooltip: seriesComponent.tooltip
                    };
                }
                series.tooltip = tooltip;
                mergeOptions(series, options?.seriesStyle);
                return series;
            })
        });
        // gridOptions.tooltip.push(tooltip)
    });
    // TODO
    if (options?.tooltip?.trigger === 'axis') {
        gridOptions.tooltip.push({
            trigger: 'axis'
        });
    }
    else {
        gridOptions.tooltip.push({});
    }
    // legend
    if (options?.legend) {
        gridOptions.legend.push(options.legend);
    }
    return gridOptions;
}
export function serializeSeriesComponent(dataset, seriesComponent, entityType, category, valueAxis, settings, options) {
    const visualMaps = [];
    // const categoryProperty = getEntityProperty(entityType, category)
    // const categoryText = getPropertyCaption(categoryProperty)
    const series = {
        id: seriesComponent.id,
        name: seriesComponent.name ?? seriesComponent.id,
        caption: seriesComponent.caption,
        type: seriesComponent.seriesType,
        // itemId: category,
        // itemName: categoryText ?? category,
        measure: seriesComponent.measure,
        datasetIndex: seriesComponent.datasetIndex,
        seriesLayoutBy: seriesComponent.seriesLayoutBy,
        stack: seriesComponent.seriesStack,
        universalTransition: settings?.universalTransition
    };
    if (seriesComponent.palette?.name) {
        const visualMap = {
            show: false,
            type: 'continuous',
            // seriesIndex: i,
            inRange: {}
        };
        const domain = seriesComponent.domain || [seriesComponent.dataMin, seriesComponent.dataMax];
        const colorFun = getChromaticScale(seriesComponent.palette.name, [0, seriesComponent.dataSize - 1], seriesComponent.palette.reverse);
        visualMap.inRange.color = range(seriesComponent.dataSize).map((i) => colorFun(i));
        visualMap.min = domain[0];
        visualMap.max = domain[1];
        visualMap.dimension = getVisualMapValueAxisIndex(dataset, seriesComponent, null);
        series.color = getLegendColorForVisualMap(visualMap.inRange.color);
        visualMaps.push(visualMap);
    }
    if (!isNil(seriesComponent.palette?.pattern)) {
        series.itemStyle = series.itemStyle ?? {};
        series.itemStyle.decal = DecalPatterns()[seriesComponent.palette.pattern];
    }
    if (!isEmpty(seriesComponent.referenceLines)) {
        assign(series, referenceLines(seriesComponent, null, options, valueAxis.orient));
        // seriesComponent.referenceLines.forEach((referenceLine) => {
        //   if (referenceLine.type) {
        //     const referenceLineData = {
        //       name: referenceLine.label
        //     }
        //     if (referenceLine.valueType === ReferenceLineValueType.fixed) {
        //       referenceLineData[valueAxis] = referenceLine.value
        //     } else if (referenceLine.valueType === ReferenceLineValueType.dynamic) {
        //       referenceLineData['type'] = referenceLine.aggregation
        //     }
        //     if (referenceLine.type === ReferenceLineType.markLine) {
        //       series.markLine = series.markLine ?? {
        //         label: {
        //           show: true,
        //           position: 'insideStartTop',
        //           formatter: '{b}:{c}'
        //         },
        //         data: []
        //       }
        //       series.markLine.data.push(referenceLineData)
        //     } else if (referenceLine.type === ReferenceLineType.markPoint) {
        //       series.markPoint = series.markPoint ?? {
        //         label: {
        //           show: true,
        //           position: 'insideStartTop',
        //           formatter: '{b}:{c}'
        //         },
        //         data: []
        //       }
        //       series.markPoint.data.push(referenceLineData)
        //     }
        //   }
        // })
    }
    if (seriesComponent.sizeMeasure) {
        visualMaps.push(getSymbolSizeVisualMap(seriesComponent.sizeMeasure, getEntityProperty(entityType, seriesComponent.sizeMeasure), dataset));
    }
    if (seriesComponent.lightnessMeasure) {
        const lightnessProperty = getEntityProperty(entityType, seriesComponent.lightnessMeasure);
        visualMaps.push(getColorLightnessVisualMap(seriesComponent.lightnessMeasure, lightnessProperty, dataset));
    }
    return {
        series,
        visualMaps
    };
}
export function getVisualMapValueAxisIndex(dataset, seriesComponent, series) {
    if (series) {
        dataset = dataset.find((item) => item.id === `--${seriesComponent.measure}`);
    }
    else if (isArray(dataset)) {
        dataset = dataset[0];
    }
    return dataset.dimensions ? indexOf(dataset.dimensions, seriesComponent.measure) : seriesComponent.measure;
}
export function getLegendColorForVisualMap(colors) {
    if (colors && colors.length > 2) {
        return new LinearGradient(0, 0, 1, 0, [
            {
                offset: 0,
                color: colors[0]
            },
            {
                offset: 0.5,
                color: colors[Math.floor(colors.length / 2)]
            },
            {
                offset: 1,
                color: colors[colors.length - 1]
            }
        ]);
    }
    else if (colors && colors.length > 1) {
        return new LinearGradient(0, 0, 1, 0, [
            {
                offset: 0,
                color: colors[0]
            },
            {
                offset: 1,
                color: colors[colors.length - 1]
            }
        ]);
    }
    else if (colors && colors.length > 0) {
        return colors[0];
    }
    return undefined;
}
export function getSymbolSizeVisualMap(measure, property, dataset) {
    // const dataMin = minBy(dataset.source, measure.measure)?.[measure.measure]
    // const dataMax = maxBy(dataset.source, measure.measure)?.[measure.measure]
    return {
        left: 'right',
        top: '10%',
        dimension: dataset.dimensions ? dataset.dimensions.indexOf(measure.measure) : measure.measure,
        // min: dataMin,
        // max: dataMax,
        itemWidth: 30,
        itemHeight: 120,
        calculable: true,
        precision: 0.1,
        text: [property.label || property.name],
        textGap: 30,
        inRange: {
            symbolSize: [10, 70]
        },
        outOfRange: {
            symbolSize: [10, 70],
            color: ['rgba(255,255,255,0.4)']
        },
        controller: {
            inRange: {
                color: ['#c23531']
            },
            outOfRange: {
                color: ['#999']
            }
        }
    };
}
export function getColorLightnessVisualMap(measure, property, dataset) {
    // const dataMin = minBy(dataset.source, measure.measure)?.[measure.measure]
    // const dataMax = maxBy(dataset.source, measure.measure)?.[measure.measure]
    return {
        left: 'right',
        bottom: '5%',
        dimension: dataset.dimensions ? dataset.dimensions.indexOf(measure.measure) : measure.measure,
        // min: dataMin,
        // max: dataMax,
        itemHeight: 120,
        text: [property.label || property.name],
        textGap: 30,
        inRange: {
            colorLightness: [0.9, 0.5]
        },
        outOfRange: {
            color: ['rgba(255,255,255,0.4)']
        },
        controller: {
            inRange: {
                color: ['#c23531']
            },
            outOfRange: {
                color: ['#999']
            }
        }
    };
}
export function measuresToSeriesComponents(measures, data, entityType, settings) {
    const tooltips = measures.filter(({ role }) => role === ChartMeasureRoleType.Tooltip);
    const _measures = measures.filter(({ role }) => !includes([ChartMeasureRoleType.Tooltip, ChartMeasureRoleType.Size, ChartMeasureRoleType.Lightness], role));
    return _measures.map((measure) => {
        const measureName = getPropertyMeasure(measure);
        const measureProperty = getEntityProperty(entityType, measure);
        const valueAxisIndex = measure.role === ChartMeasureRoleType.Axis2 ? 1 : 0;
        const dataNotNull = data.filter((item) => !isNil(item[measureName]));
        const minItem = minBy(dataNotNull, measureName);
        const maxItem = maxBy(dataNotNull, measureName);
        return {
            ...measure,
            // TODO 如何设置规律的 series ID 编码
            // id: settings?.universalTransition ? getPropertyMeasure(measure) : null,
            id: getPropertyMeasure(measure),
            name: measureProperty?.label,
            label: measureProperty?.label,
            seriesType: measure.shapeType,
            property: measureProperty,
            dataMin: minItem?.[measureName],
            dataMax: maxItem?.[measureName],
            dataSize: data.length,
            valueAxisIndex,
            tooltip: tooltips.map(({ measure }) => measure),
            sizeMeasure: measures.find(({ role }) => role === ChartMeasureRoleType.Size),
            lightnessMeasure: measures.find(({ role }) => role === ChartMeasureRoleType.Lightness)
        };
    });
}
export function dimensionToSeriesComponent(name, dimension, measures, data, entityType, settings) {
    const property = getEntityProperty(entityType, dimension);
    const tooltips = measures.filter(({ role }) => role === ChartMeasureRoleType.Tooltip);
    return {
        // ...omit(dimension, 'role'),
        id: settings?.universalTransition ? getPropertyHierarchy(dimension) : null,
        name: name ?? property.label ?? property.name,
        label: property?.label,
        property,
        dataSize: data.length,
        tooltip: tooltips.map(({ measure }) => measure),
        sizeMeasure: measures.find(({ role }) => role === ChartMeasureRoleType.Size),
        lightnessMeasure: measures.find(({ role }) => role === ChartMeasureRoleType.Lightness)
    };
}
/**
 *
 * @param chartAnnotation
 * @param entityType
 * @param items
 * @param chartOptions
 * @param locale
 * @returns
 */
export function getCoordinateSystem(chartAnnotation, entityType, items, chartOptions, locale) {
    const [categoryOrient, valueOrient] = axisOrient(chartAnnotation.chartType.orient);
    const category = getChartCategory(chartAnnotation);
    const category2 = getChartCategory2(chartAnnotation);
    if (chartAnnotation.chartType.type === 'Scatter') {
        const valueAxises = getValueAxis(chartAnnotation, entityType, chartOptions, locale);
        return {
            categoryAxis: { orient: categoryOrient, axis: [valueAxises[0]] },
            valueAxis: { orient: valueOrient, axis: [valueAxises[1]] }
        };
    }
    else {
        let valueAxis = null;
        // 设置维度轴值
        const categoryAxis = getCategoryAxis(items, category, getEntityHierarchy(entityType, category), chartOptions);
        categoryAxis.orient = categoryOrient;
        if (category2) {
            valueAxis = [getCategoryAxis(items, category2, getEntityHierarchy(entityType, category2), chartOptions)];
        }
        else {
            valueAxis = getValueAxis(chartAnnotation, entityType, chartOptions, locale);
        }
        valueAxis.orient = valueOrient;
        return {
            categoryAxis: { orient: categoryOrient, axis: [categoryAxis] },
            valueAxis: { orient: valueOrient, axis: valueAxis }
        };
    }
}
//# sourceMappingURL=cartesian.js.map